﻿using Styx;
using Styx.Common;
using Styx.Plugins;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Styx.TreeSharp;
using Styx.Pathing;
using CommonBehaviors.Actions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Media;
using Styx.CommonBot;
using Styx.CommonBot.POI;
using System.Diagnostics;

namespace TheLegionLooter
{
    public class TheLegionLooter : HBPlugin
    {
        public override string Author => "ElPargo";
        public override string Name => "The Legion Looter";
        public override Version Version => new Version(1, 0);
        public override string ButtonText => "TLL Settings";
        public override bool WantButton => true;
        public LocalPlayer Me => StyxWoW.Me;
        private static readonly List<Crystal> Crystals = new List<Crystal>();
        public static string LastInfoLogMsg;
        private Composite _task;

        public override void OnButtonPress()
        {
            SettingsForm frm = new SettingsForm();
            frm.ShowDialog();
            TLLSet.Instance.Save();
        }

        internal async Task<bool> MainTask()
        {
            var scrap = GetFlowerScrap();
            if (scrap != null && !Me.Combat)
            {
                Navigator.MoveTo(scrap.Location);
                return true;
            }
            if (IsFoxPresent() && !Me.Combat)
            {
                return true;
            }
            return false;
        }

        private static bool IsFoxPresent()
        {
            using (StyxWoW.Memory.AcquireFrame())
            {
                IEnumerable<WoWUnit> mobsOfInterestQuery =
                    from wowUnit in ObjectManager.GetObjectsOfType<WoWUnit>()
                    where
                        (int)wowUnit.Entry == 98235
                        && wowUnit.IsAlive
                        && !wowUnit.TaggedByOther
                    orderby
                      wowUnit.DistanceSqr
                    select
                      wowUnit;
                return mobsOfInterestQuery.ToList().Any();
            }
        }

        private static WoWAreaTrigger GetFlowerScrap()
        {
            IEnumerable<WoWAreaTrigger> t =
            from trigger in ObjectManager.GetObjectsOfType<WoWAreaTrigger>()
            where
                (int)trigger.Entry == 9756
            orderby
                trigger.DistanceSqr
            select
                trigger;

            return t.FirstOrDefault();
        }

        private static bool IsEntryACrystal(uint id)
        {
            foreach (var item in Crystals)
            {
                if (item.Id == id)
                { return true; }
            }
            return false;
        }

        public override void OnEnable()
        {
            base.OnEnable();
            Logging.Write("The Legion Looter: Enabled!");

            //Add Specific containers here.
            Crystals.Clear();
            Crystals.Add(new Crystal() { Id = 252774, Name = "Ancient Mana Crystal" });
            Crystals.Add(new Crystal() { Id = 245941, Name = "Warp Cache" });
            Crystals.Add(new Crystal() { Id = 252432, Name = "Shimmering Ancient Mana Cluster" });
            Crystals.Add(new Crystal() { Id = 252446, Name = "Shimmering Ancient Mana Cluster" });
            Crystals.Add(new Crystal() { Id = 252447, Name = "Shimmering Ancient Mana Cluster" });
            Crystals.Add(new Crystal() { Id = 252448, Name = "Shimmering Ancient Mana Cluster" });
            Crystals.Add(new Crystal() { Id = 252449, Name = "Shimmering Ancient Mana Cluster" });
            Crystals.Add(new Crystal() { Id = 252450, Name = "Shimmering Ancient Mana Cluster" });
            Crystals.Add(new Crystal() { Id = 254006, Name = "Enchanted Burial Urn" });
            Crystals.Add(new Crystal() { Id = 254009, Name = "Volatile Leyline Crystal" });
            Crystals.Add(new Crystal() { Id = 254008, Name = "Kyrtos's Research Notes" });
            Crystals.Add(new Crystal() { Id = 248407, Name = "Kel'danath's Manaflask" });
            Crystals.Add(new Crystal() { Id = 254023, Name = "Arcane Power Unit" });
            Crystals.Add(new Crystal() { Id = 95958, Name = "Floating Treasure" });
            Crystals.Add(new Crystal() { Id = 245479, Name = "Battered Chest" });
            Crystals.Add(new Crystal() { Id = 240638, Name = "Disputed Treasure" });
            Crystals.Add(new Crystal() { Id = 252408, Name = "Ancient Mana Shard" });
            Crystals.Add(new Crystal() { Id = 252772, Name = "Ancient Mana Chunk" });

            _task = new ActionRunCoroutine(r => MainTask());
            TreeHooks.Instance.InsertHook("InGame_Check", 0, _task);
			
			InfoLog("By ElPargo | Version " + Version + " | [Enabled]");
        }

        public override void OnDisable()
        {
            base.OnDisable();
            TreeHooks.Instance.RemoveHook("InGame_Check", _task);
			InfoLog("[Disabled]");
        }

		public void InfoLog(string message, params object[] args)
        {
            if (message == LastInfoLogMsg) { return; }
            
            Logging.Write(Color.FromRgb(200, 255, 0), "[The Legion Looter]: {0}", message);
            LastInfoLogMsg = message;
        }

        public static void DebugLog(string message, params object[] args)
        { Logging.WriteQuiet(message, args); }

        public override void Pulse()
        {
            if (!StyxWoW.IsInGame || Me == null || !Me.IsValid || !Me.IsAlive || Me.OnTaxi || Me.InVehicle) { return; }

            var mob = ObjectManager.GetObjectsOfTypeFast<WoWUnit>()
                .Where(u => u.IsAlive && u.Classification == WoWUnitClassificationType.Rare)
                .OrderBy(u => u.Distance)
                .FirstOrDefault();
            if (mob != null && !Me.IsCasting && mob.Distance <= TLLSet.Instance.MaxRange && TLLSet.Instance.FindRares && !Blacklist.Contains(mob, BlacklistFlags.All))
            {
                InfoLog(mob.GetObjectName() + " Detected: Moving to Kill!");
                BotPoi.Current = new BotPoi(mob, PoiType.Kill, NavType.Fly);
            }

            var chest = ObjectManager.GetObjectsOfTypeFast<WoWObject>()
                .Where(o => IsEntryACrystal(o.Entry) || o.Name.Contains("Treasure Chest"))
                .OrderBy(o => o.Distance)
                .FirstOrDefault();
            if (chest != null && mob == null && !Me.IsCasting && chest.Distance <= TLLSet.Instance.MaxRange && TLLSet.Instance.FindChests && !Blacklist.Contains(chest, BlacklistFlags.All))
            {
                if ((chest.Entry == 252408 || chest.Entry == 252772) && !TLLSet.Instance.FindMana) { return; }

                InfoLog(chest.Guid + ": " + chest.GetObjectName() + " Detected: Moving to Loot!");
                BotPoi.Current = new BotPoi(chest, PoiType.Harvest, NavType.Fly);
            }
        }
    }

    public class Crystal
    {
        public uint Id { get; set; }
        public string Name { get; set; }
    }
}
